@@ -1,12 +1,13 @@
+import logging
 import os
 import pickle
 import sys
-import logging
+
 import cloudpickle
-from .db import job_db_factory
 
 from pyabc.sge.execution_contexts import NamedPrinter
 
+from .db import job_db_factory
 
 logger = logging.getLogger("ABC.SGE")
 
@@ -36,13 +37,19 @@ with open(os.path.join(tmp_path, 'ExecutionContext.pickle'), 'rb') as my_file:
 results_array = []
 for element in array:
     try:
-        with NamedPrinter(tmp_path, job_nr), \
-             ExecutionContext(tmp_path, job_nr):
+        with NamedPrinter(tmp_path, job_nr), ExecutionContext(
+            tmp_path, job_nr
+        ):
             single_result = function(element)
     except Exception as e:
         logger.error(
             "execute_sge_array_job: Exception in sge-worker path=",
-            tmp_path, 'jobnr=', job_nr, "exception", e)
+            tmp_path,
+            'jobnr=',
+            job_nr,
+            "exception",
+            e,
+        )
         single_result = e
     else:
         pass
@@ -50,6 +57,7 @@ for element in array:
         results_array.append(single_result)
 
 # store result
-with open(os.path.join(tmp_path, 'results',
-                       job_nr + '.result'), 'wb') as my_file:
+with open(
+    os.path.join(tmp_path, 'results', job_nr + '.result'), 'wb'
+) as my_file:
     cloudpickle.dump(results_array, my_file)
